/* SPDX-License-Identifier: GPL-2.0-or-later */
/*
 * Copyright 1997 Transmeta Corporation - All Rights Reserved
 */

#ifndef _LINUX_AUTO_FS_H
#define _LINUX_AUTO_FS_H

#include <linux/fs.h>
#include <linux/ioctl.h>
#include <uapi/linux/auto_fs.h>
#endif /* _LINUX_AUTO_FS_H */
